@@ -8,7 +8,7 @@ from dataclasses import dataclass
 from typing import Callable, TypeVar, Any, Optional, Union, cast, List
 from gitlab.v4.objects import Project
 from git import Repo
-from datetime import datetime
+from datetime import datetime, date
 
 from ..utility_base import UtilityBase
 from ..logger import Logger, LogWrapper
@@ -46,6 +46,15 @@ class CommitMetadata:
     commit_id: str
     commit_date: str
 
+    # Optional change magnitude fields
+    lines_added: Optional[int] = None
+    lines_deleted: Optional[int] = None
+    total_lines_changed: Optional[int] = None
+    files_changed: Optional[int] = None
+    changed_files: Optional[List[str]] = None
+    directories_touched: Optional[List[str]] = None
+    entropy: Optional[float] = None
+
 
 def handle_gitlab_errors(
     default_return: Optional[T] = None,
@@ -101,6 +110,7 @@ class GitlabClient(UtilityBase):
         url: GitLab instance URL.
         private_token: Personal/private access token.
         verbose: Enable verbose logging.
+        ssl_verify: Verify SSL certificates.
         logger: Optional logger instance.
         log_level: Logging level.
         timeout: API call timeout (seconds).
@@ -112,6 +122,7 @@ class GitlabClient(UtilityBase):
         url: str,
         private_token: str,
         verbose: bool = False,
+        ssl_verify: bool = True,
         logger: Optional[Union[Logger, LogWrapper]] = None,
         log_level: Optional[int] = None,
         timeout: float = 50,
@@ -124,7 +135,7 @@ class GitlabClient(UtilityBase):
         self.retries = retries
         self.large_repo_threshold = large_repo_threshold
 
-        self.gl = gitlab.Gitlab(url, private_token=private_token, timeout=timeout, retry_transient_errors=True)
+        self.gl = gitlab.Gitlab(url, private_token=private_token, timeout=timeout, retry_transient_errors=True, ssl_verify=ssl_verify)
         self.gl.auth()
 
     def _warn_large(self, resource: Any, project_path: str, label: str) -> None:
@@ -134,6 +145,16 @@ class GitlabClient(UtilityBase):
         if count > self.large_repo_threshold:
             self._log_warning(f"Project '{project_path}' has {count}+ {label}. all=True may be slow.")
 
+    def _to_iso8601(self, dt: Union[str, datetime, date]) -> str:
+        """Accept str (assumed already ISO-8601), date, or datetime. Naive treated as UTC."""
+        if isinstance(dt, str):
+            return dt
+        if isinstance(dt, date) and not isinstance(dt, datetime):
+            dt = datetime.combine(dt, datetime.min.time())
+        if dt.tzinfo is None:
+            return dt.strftime("%Y-%m-%dT%H:%M:%SZ")
+        return dt.astimezone().strftime("%Y-%m-%dT%H:%M:%SZ")
+
     @staticmethod
     def _parse_committed_date(commit_info: dict) -> datetime:
         """Parse the committed date from a commit info dict."""
@@ -333,19 +354,48 @@ class GitlabClient(UtilityBase):
         diffs = comparison.get('diffs', [])
         changed_files = {diff['new_path'] for diff in diffs}
         return list(changed_files)
+    
+    @handle_gitlab_errors(default_return=[])
+    def list_changed_files_for_commit(
+        self,
+        project_path: str,
+        commit_id: str
+    ) -> list[str]:
+        """
+        List files changed between two commits (SHAs, tags, or branch names).
+
+        Args:
+            project_path: Project path or ID.
+            commit_id: Base (older) commit/ref.
+
+        Returns:
+            Sorted list of unique changed file paths.
+        """
+        if not (project := self.get_project(project_path)):
+            return []
+
+        commit = project.commits.get(commit_id)
+        diffs = commit.diff(get_all=True)
+
+        # Extract changed file paths
+        changed_files = [d['new_path'] for d in diffs]
+
+        return sorted(changed_files)
 
     @handle_gitlab_errors(default_return=None)
     def get_commit_metadata(
         self, 
         project_path: str, 
-        commit_hash: str
+        commit_hash: str,
+        include_change_stats: bool = False
     ) -> Optional[CommitMetadata]:
         """
-        Get metadata for a single commit by hash.
+        Get metadata for a single commit by hash. Optionally include change magnitude stats.
 
         Args:
             project_path (str): Project path or ID.
             commit_hash (str): Commit hash or short hash.
+            include_change_stats (bool): If True, include diff/size statistics.
 
         Returns:
             CommitMetadata or None.
@@ -358,8 +408,8 @@ class GitlabClient(UtilityBase):
         except Exception:
             self._log_exception(f"Error occurred while requesting commit metadata for `{commit_hash}`")
             return None
-        
-        return CommitMetadata(
+
+        meta = CommitMetadata(
             commit_short_id=commit.short_id,
             commit_title=commit.title,
             commit_author_name=commit.author_name,
@@ -367,41 +417,96 @@ class GitlabClient(UtilityBase):
             commit_date=commit.committed_date
         )
 
+        if include_change_stats:
+            try:
+                diffs = commit.diff(get_all=True)
+                files = [d['new_path'] for d in diffs]
+                stats = getattr(commit, 'stats', {}) or {}
+                additions = stats.get('additions')
+                deletions = stats.get('deletions')
+                total = stats.get('total')  # GitLab provides this (additions + deletions)
+
+                # Directory breadth
+                dirs = sorted({f.split('/')[0] for f in files if '/' in f})
+
+                # Entropy (distribution of changed lines per file, normalized)
+                per_file_changes = []
+                for d in diffs:
+                    # Fallback: approximate by counting + lines in diff (cheap heuristic)
+                    patch = d.get('diff', '') or ''
+                    added = patch.count('\n+')
+                    removed = patch.count('\n-')
+                    per_file_changes.append(max(1, added + removed))
+                entropy = None
+                if per_file_changes:
+                    import math
+                    total_lines = sum(per_file_changes)
+                    probs = [c / total_lines for c in per_file_changes]
+                    raw_entropy = -sum(p * math.log2(p) for p in probs if p > 0)
+                    entropy = raw_entropy / math.log2(len(per_file_changes)) if len(per_file_changes) > 1 else 0.0
+
+                meta.lines_added = additions
+                meta.lines_deleted = deletions
+                meta.total_lines_changed = total
+                meta.files_changed = len(files)
+                meta.changed_files = files
+                meta.directories_touched = dirs
+                meta.entropy = entropy
+            except Exception as e:
+                self._log_warning(f"Could not compute change stats for `{commit_hash}`: {e}")
+
+        return meta
+
     @handle_gitlab_errors(default_return=[])
     def list_commits(
-        self, project_path: str, branch: str, max_commits: Optional[int] = None
+        self,
+        project_path: str,
+        branch: str,
+        max_commits: Optional[int] = None,
+        since: Optional[Union[str, datetime, date]] = None,
+        until: Optional[Union[str, datetime, date]] = None,
     ) -> list[CommitMetadata]:
         """
-        List commits on a branch.
+        List commits on a branch (optionally filtered by date range).
 
         Args:
-            project_path (str): Project path or ID.
-            branch (str): Branch to list commits from.
-            max_commits (Optional, int): Max commits to return. If None, return all.
-
-        Returns:
-            List of CommitMetadata.
+            project_path: Project path or ID.
+            branch: Branch to list commits from.
+            max_commits: If provided, limit mapped results (per_page size); omit for full range.
+            since: Include commits at/after this timestamp/date (ISO-8601, datetime, or date).
+            until: Include commits up to this timestamp/date (inclusive per GitLab semantics).
         """
         if not (project := self.get_project(project_path)):
             return []
-        
+
+        params: dict[str, Any] = {"ref_name": branch}
+        if since:
+            params["since"] = self._to_iso8601(since)
+        if until:
+            params["until"] = self._to_iso8601(until)
+
         if max_commits is not None:
-            commits = project.commits.list(ref_name=branch, per_page=max_commits)
+            commits = project.commits.list(per_page=max_commits, **params)
         else:
             self._warn_large(project.commits, project_path, "commits")
-            commits = project.commits.list(ref_name=branch, all=True)
+            commits = project.commits.list(all=True, **params)
 
-        return [
+        result = [
             CommitMetadata(
-                commit_short_id=commit.short_id,
-                commit_title=commit.title,
-                commit_author_name=commit.author_name,
-                commit_id=commit.id,
-                commit_date=commit.committed_date
+                commit_short_id=c.short_id,
+                commit_title=c.title,
+                commit_author_name=c.author_name,
+                commit_id=c.id,
+                commit_date=c.committed_date
             )
-            for commit in commits
+            for c in commits
         ]
 
+        if max_commits is not None:
+            result = result[:max_commits]
+
+        return result
+
     @handle_gitlab_errors(default_return=[])
     def list_unique_commits_on_branch(
         self, project_path: str, base_branch: str, compare_branch: str
@@ -489,9 +594,10 @@ class GitlabClient(UtilityBase):
         """
         repo = Repo(local_path)
 
-        repo.git.fetch()
+        repo.git.fetch('--all')
         try:
             repo.git.checkout(branch)
+            repo.git.pull('--ff-only')
         except Exception:
             repo.git.checkout('-b', branch, f'origin/{branch}')
 
