@@ -1,6 +1,7 @@
 import aiohttp
 import asyncio
 import logging
+import ssl
 
 from typing import Optional, Dict, Any, List, Union
 from collections import defaultdict
@@ -22,6 +23,7 @@ class JiraClient(UtilityBase):
             jira_url: str,
             retry_cnt: int = 1,
             verbose: bool = False,
+            ssl_verify: bool = True,
             log_urls: bool = False,
             logger: Optional[Union[logging.Logger, Logger, LogWrapper]] = None,
             log_level: Optional[int] = None
@@ -34,6 +36,7 @@ class JiraClient(UtilityBase):
             jira_url (str): Base URL of the Jira instance.
             retry_cnt (int, optional): Number of retries in case of a failed request.
             verbose (bool, optional): If True, debug messages are logged during the operations. Exceptions, Errors and Warnings are always logged.
+            ssl_verify (bool, optional): If False, SSL certificate verification is disabled.
             log_urls (bool, optional): If True, all the requested URLs are logged.
             logger (Optional[Union[logging.Logger, Logger, LogWrapper]], optional): Logger instance. If not provided, a default logger is used.
             log_level (Optional[int], optional): Log level. If not provided, INFO level will be used for logging.
@@ -50,11 +53,13 @@ class JiraClient(UtilityBase):
         self.session: Optional[aiohttp.ClientSession] = None  # Session is not created yet
 
         self.jira_url = jira_url
+        self.base_project_url = f"{jira_url}/rest/api/2/project"
         self.base_issue_url = f"{jira_url}/rest/api/2/issue"
         self.base_search_url = f"{jira_url}/rest/api/2/search"
         self.base_board_url = f"{jira_url}/rest/agile/1.0/board"
 
         self.retry_cnt = retry_cnt
+        self.ssl_verify = ssl_verify
                 
     async def _fetch_json(self, url: str) -> Optional[Dict[str, Any]]:
         """
@@ -132,7 +137,13 @@ class JiraClient(UtilityBase):
     async def start_session(self):
         """Explicitly initialize the aiohttp session. Must be called once before making requests."""
         if self.session is None:
-            self.session = aiohttp.ClientSession()
+            if not self.ssl_verify:
+                ssl_context = ssl.create_default_context()
+                ssl_context.check_hostname = False
+                ssl_context.verify_mode = ssl.CERT_NONE
+                self.session = aiohttp.ClientSession(connector=aiohttp.TCPConnector(ssl=ssl_context))
+            else:
+                self.session = aiohttp.ClientSession()
         else:
             self._log("Session already initialized. Skipping.")
 
@@ -367,6 +378,28 @@ class JiraClient(UtilityBase):
         self._log(f"Read {len(sprint_list)} sprints for board `{board_id}`")
         return sprint_list
     
+    async def read_project_release_list(self, project_id: str) -> List[Dict[str, Any]] | None:
+        """
+        Asynchronously retrieve the release list (fix versions) of a project.
+
+        Constructs the request URL using the project ID and fetches the JSON response.
+
+        Args:
+            project_id (str): The ID of the Jira project.
+
+        Returns:
+            Optional[List[Dict[str, Any]]]: The read version objects or None.
+        """
+        url = f"{self.base_project_url}/{project_id}/versions"
+        response = await self._fetch_json(url)
+
+        if response and len(response) > 0:
+            return response
+        else:
+            self._log_error(f"Could not retrieve version list for project `{project_id}`")
+                
+        return None
+
     async def send_request(
         self,
         method: str,
